"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Setting_1 = require("@joplin/lib/models/Setting");
const test_utils_1 = require("@joplin/lib/testing/test-utils");
const Folder_1 = require("@joplin/lib/models/Folder");
const testUtils_1 = require("../../../testUtils");
const eventManager_1 = require("@joplin/lib/eventManager");
describe('JoplinSettings', () => {
    beforeEach(async () => {
        await (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        await (0, test_utils_1.switchClient)(1);
    });
    afterEach(async () => {
        await (0, test_utils_1.afterEachCleanUp)();
    });
    test('should listen to setting change event', async () => {
        const service = (0, testUtils_1.newPluginService)();
        const pluginScript = (0, testUtils_1.newPluginScript)(`
			joplin.plugins.register({
				onStart: async function() {
					await joplin.settings.registerSettings({
						'myCustomSetting1': {
							value: 1,
							type: 1,
							public: true,
							label: 'My Custom Setting 1',
						},
						'myCustomSetting2': {
							value: 2,
							type: 1,
							public: true,
							label: 'My Custom Setting 2',
						}
					})

					joplin.settings.onChange((event) => {
						joplin.data.post(['folders'], null, { title: JSON.stringify(event.keys) });
					});
				},
			});
		`);
        const plugin = await service.loadPluginFromJsBundle('', pluginScript);
        await service.runPlugin(plugin);
        Setting_1.default.setValue('plugin-org.joplinapp.plugins.PluginTest.myCustomSetting1', 111);
        Setting_1.default.setValue('plugin-org.joplinapp.plugins.PluginTest.myCustomSetting2', 222);
        // Also change a global setting, to verify that the plugin doesn't get
        // notifications for non-plugin related events.
        Setting_1.default.setValue('locale', 'fr_FR');
        Setting_1.default.emitScheduledChangeEvent();
        await (0, test_utils_1.waitForFolderCount)(1);
        const folder = (await Folder_1.default.all())[0];
        const settingNames = JSON.parse(folder.title);
        settingNames.sort();
        expect(settingNames.join(',')).toBe('myCustomSetting1,myCustomSetting2');
        await service.destroy();
    });
    test('should de-register settings change listeners when a plugin is unloaded', async () => {
        const service = (0, testUtils_1.newPluginService)();
        const pluginScript = (0, testUtils_1.newPluginScript)(`
			joplin.plugins.register({
				onStart: async function() {
					await joplin.settings.registerSettings({
						'test-setting': {
							value: 1234,
							type: 1,
							public: false,
							label: 'Test',
						}
					});

					// Register 8 listeners to improve test reliability in the case
					// where listeners are added/removed from other sources.
					for (let i = 0; i < 8; i++) {
						await joplin.settings.onChange((event) => { });
					}
				},
			});
		`);
        const plugin = await service.loadPluginFromJsBundle('', pluginScript);
        await service.runPlugin(plugin);
        const listenerCounter = eventManager_1.default.listenerCounter_(eventManager_1.EventName.SettingsChange);
        plugin.onUnload();
        expect(listenerCounter.getCountRemoved()).toBeGreaterThanOrEqual(8);
    });
    test('should allow registering multiple settings', async () => {
        const service = (0, testUtils_1.newPluginService)();
        const pluginScript = (0, testUtils_1.newPluginScript)(`
			joplin.plugins.register({
				onStart: async function() {
					await joplin.settings.registerSettings({
						'myCustomSetting1': {
							value: 1,
							type: 1,
							public: true,
							label: 'My Custom Setting 1',
						},
						'myCustomSetting2': {
							value: 2,
							type: 1,
							public: true,
							label: 'My Custom Setting 2',
						}
					})
				},
			});
		`);
        const plugin = await service.loadPluginFromJsBundle('', pluginScript);
        await service.runPlugin(plugin);
        expect(Setting_1.default.value('plugin-org.joplinapp.plugins.PluginTest.myCustomSetting1')).toBe(1);
        expect(Setting_1.default.value('plugin-org.joplinapp.plugins.PluginTest.myCustomSetting2')).toBe(2);
        await service.destroy();
    });
});
//# sourceMappingURL=JoplinSettings.js.map